@@ -1,5 +1,5 @@
 /* GKRELLMWireless
-|  Copyright (C) 2000-2001 Sjoerd Simons
+|  Copyright (C) 2000-2002 Sjoerd Simons
 |
 |  Author:  Sjoerd Simons sjoerd@gkrellm.luon.net
 |  Latest versions might be found at:  http://gkrellm.luon.net
@@ -20,6 +20,7 @@
 
 #include "wireless.h"
 
+static GkrellmMonitor *monitor;
 static gint panel_style_id;
 static wcard_t *cards = NULL;
 static GtkWidget *PanelBox;
@@ -39,9 +40,10 @@ new_wcard(gchar *interface,int found, int flags) {
    newcard->level_panel = NULL;
    newcard->link_panel = NULL;
    newcard->noise_panel = NULL;
+   newcard->bitrate_panel = NULL;
 
-   if (found) newcard->flags = (0 | SHOW | SHOW_LINK | SHOW_LEVEL | SHOW_NOISE);
-   else  /* a card wich is not detected, but from a config file */
+   if (found) newcard->flags = (0 | SHOW | SHOW_LINK | SHOW_LEVEL | SHOW_NOISE | SHOW_BITRATE);
+   else  /* a card which is not detected, but from a config file */
      newcard->flags = flags & ~ACTIVE;
 
    if (cards == NULL) cards = newcard;
@@ -265,17 +267,26 @@ get_link_quality(wcard_t *card, float *quality, float *level, float *noise) {
   }
   return FALSE;
 }
+
+static int
+get_bitrate(wcard_t *card, int *maxbitrate, int *bitrate) {
+  *maxbitrate = 0;
+  *bitrate = 0;
+  return FALSE;
+}
 #endif
 
 #ifdef __linux__
 /* Linux Specific*/
+
+#define WIRELESS "/proc/net/wireless"
 static gint
 find_wlancard(void) {
   FILE *procfile;
   int ret = FALSE;
   char iface[5],procread[256], *c;
   
-  if ((procfile = fopen("/proc/net/wireless","r")) == NULL) return FALSE;
+  if ((procfile = fopen(WIRELESS,"r")) == NULL) return FALSE;
 
   /* 2 lines header */
   fgets(procread,sizeof(procread),procfile);
@@ -293,24 +304,40 @@ find_wlancard(void) {
   return ret;
 }
  
+float get_next_fl(char **string) {
+  char *c;
+  float ret;
+  c = *string;
+  while (!isdigit(*c) && *c != '\0') c++;
+  ret = atof(*string);
+  while(!isspace(*c) && *c != '\0') c++;
+  *string = c;
+  return ret;
+}
+
+
 static int
 get_link_quality(wcard_t *card, float *link, float *level, float *noise) {
   FILE *procfile;
-  char iface[5], procread[256], *c;
+  char procread[256], *c;
 
-  if ((procfile = fopen("/proc/net/wireless","r")) != NULL) {
+  if ((procfile = fopen(WIRELESS,"r")) != NULL) {
     /* 2 lines header */
     fgets(procread,sizeof(procread),procfile);
     fgets(procread,sizeof(procread),procfile);
 
     /* search for our card */
     while (fgets(procread,sizeof(procread),procfile) != NULL) {
-      sscanf(procread,"%s %*s %f %f %f %*d %*d %*d",iface,link,level,noise);
-      c = strstr(iface,":");
-      *c = '\0';
-      if (!strcmp(iface,card->ifname)) {
-	fclose(procfile);
-	return TRUE;
+      c = procread;
+      while (isspace(*c)) c++;
+      if (!strncmp(c,card->ifname,strlen(card->ifname))) {
+        while (!isspace(*c) && *c != '\0') c++;
+                 get_next_fl(&c); /* status thingie */
+         *link = get_next_fl(&c);
+        *level = get_next_fl(&c) - 256;
+        *noise = get_next_fl(&c) -256;
+      	fclose(procfile);
+	      return TRUE;
       }
     }
     fclose(procfile);
@@ -318,11 +345,91 @@ get_link_quality(wcard_t *card, float *link, float *level, float *noise) {
   del_wcard(card);
   return FALSE;
 }
-#endif
+
+/* the next three functions (iw_sockets_open, iw_get_ext,
+   iw_get_range_info) were copied and modified from iwlib.c which is
+   part of the wireless-tools package by Jean Tourrilhes */
+
+static int
+iw_sockets_open(void) {
+  static const int families[] = {
+    AF_INET, AF_IPX, AF_AX25, AF_APPLETALK
+  };
+  unsigned int i;
+  int sock;
+  
+  for(i = 0; i < (sizeof(families) / sizeof(int)); ++i) {
+    sock = socket(families[i], SOCK_DGRAM, 0);
+    if(sock >= 0)
+      return sock;
+  }
+  
+  return -1;
+}
+
+static inline int
+iw_get_ext(int skfd, char *ifname, int request, struct iwreq *pwrq) {
+  strncpy(pwrq->ifr_name, ifname, IFNAMSIZ);
+  return ioctl(skfd, request, pwrq);
+}
+
+static int
+iw_get_range_info(int	skfd, char *ifname, struct iw_range *range) {
+  struct iwreq wrq;
+  /* ask for the range info, but get it in a buffer that's much larger
+     than necessary, because the wireless stuff maintains almost no
+     binary-level backward compatibility, so the data passed back
+     could actually be larger than expected, in a different format
+     than expected :-/ */
+  char buffer[sizeof(struct iw_range) * 2];
+  memset(buffer, 0, sizeof(buffer));
+  wrq.u.data.pointer = (caddr_t)buffer;
+  wrq.u.data.length = sizeof(buffer);
+  wrq.u.data.flags = 0;
+  if(iw_get_ext(skfd, ifname, SIOCGIWRANGE, &wrq) < 0)
+    return -1;
+  memcpy((char *)range, buffer, sizeof(struct iw_range));
+  return 0;
+}
+
+static int
+get_bitrate(wcard_t *card, int *maxbitrate, int *bitrate) {
+  int skfd;
+  struct iw_range range;
+  struct iwreq wrq;
+  int i;
+  
+  /* open a socket, any socket */
+  if((skfd = iw_sockets_open()) < 0)
+    return FALSE;
+  
+  /* get bitrate range info */
+  if((iw_get_ext(skfd, card->ifname, SIOCGIWRATE, &wrq) < 0)
+     || (iw_get_range_info(skfd, card->ifname, &range) < 0)) {
+    close(skfd);
+    return FALSE;
+  }
+  
+  close(skfd);
+  
+  /* check that there are a reasonable number of bitrates */
+  if((range.num_bitrates <= 0) || (range.num_bitrates > IW_MAX_BITRATES))
+    return FALSE;
+  
+  /* figure out the current bitrate and the largest bitrate */
+  *bitrate = wrq.u.bitrate.value;
+  *maxbitrate = range.bitrate[0];
+  for(i = 1; i < range.num_bitrates; ++i)
+    if(*maxbitrate < range.bitrate[i])
+      *maxbitrate = range.bitrate[i];
+  
+  return TRUE;
+}
+#endif /* __linux__ */
 
 /* end of system specific code */
 static gint
-panel_expose_event(GtkWidget *widget, GdkEventExpose *event,Panel *panel) {
+panel_expose_event(GtkWidget *widget, GdkEventExpose *event, GkrellmPanel *panel) {
   gdk_draw_pixmap(widget->window,
     widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
     panel->pixmap, event->area.x, event->area.y, event->area.x, event->area.y,
@@ -332,23 +439,21 @@ panel_expose_event(GtkWidget *widget, GdkEventExpose *event,Panel *panel) {
 }
 
 static void 
-del_panel(Panel *panel) {
+del_panel(GkrellmPanel *panel) {
   if (panel == NULL) return;
-  gkrellm_monitor_height_adjust(- panel->h);
   gkrellm_destroy_decal_list(panel);
   gkrellm_destroy_krell_list(panel);
-  gkrellm_destroy_panel(panel);
+  gkrellm_panel_destroy(panel);
   gkrellm_pack_side_frames();
-  free(panel);
 }
 
 static void
-create_panel(Panel **rpanel,Decal **text, int fullscale,int create) {
-  Krell *k;
-  Style *style;
-  GdkImlibImage *krell_image;
+create_panel(GkrellmPanel **rpanel, GkrellmDecal **text, int fullscale,int create) {
+  GkrellmKrell *k;
+  GkrellmStyle *style;
+  GkrellmPiximage  *krell_image;
   int first_create = 0 ;
-  Panel *panel = *rpanel;
+  GkrellmPanel *panel = *rpanel;
 
   if (panel == NULL) {
       panel = gkrellm_panel_new0();
@@ -359,25 +464,24 @@ create_panel(Panel **rpanel,Decal **text, int fullscale,int create) {
   }
   style = gkrellm_meter_style(panel_style_id);
 
-  krell_image = gkrellm_krell_meter_image(panel_style_id);
+  krell_image = gkrellm_krell_meter_piximage(panel_style_id);
   k = gkrellm_create_krell(panel, krell_image, style);
-  k->full_scale = fullscale;
+  gkrellm_set_krell_full_scale(k, fullscale, 1);
 
   panel->textstyle = gkrellm_meter_textstyle(panel_style_id);
 
   *text = gkrellm_create_decal_text(panel,"0"
                                           ,panel->textstyle,style,-1,-1,-1);
-  gkrellm_configure_panel(panel,NULL,style);
-  gkrellm_create_panel(PanelBox,panel ,gkrellm_bg_meter_image(panel_style_id)); 
+  gkrellm_panel_configure(panel,NULL,style);
+  gkrellm_panel_create(PanelBox, monitor, panel); 
 
   if (first_create || create) {
-      gkrellm_monitor_height_adjust(panel->h);
-      gtk_signal_connect(GTK_OBJECT(panel->drawing_area),"expose_event",
-                         (GtkSignalFunc)panel_expose_event,panel); 
+      g_signal_connect(GTK_OBJECT(panel->drawing_area), "expose_event",
+                         G_CALLBACK(panel_expose_event),panel);
   }
 
   gkrellm_draw_decal_text(panel,*text,"wireless",-10);
-  gkrellm_draw_layers(panel);
+  gkrellm_draw_panel_layers(panel);
 
   gkrellm_pack_side_frames();
 
@@ -386,6 +490,7 @@ create_panel(Panel **rpanel,Decal **text, int fullscale,int create) {
 
 static void
 reset_panel(int create) {
+  int bitrate, mbitrate;
   /* (re)create all the needed panels */
   /* if create then gkrellm is newly build (theme switch or whatever) */
   /* else it's called by the plugin itself (new card, changed config) */
@@ -395,6 +500,7 @@ reset_panel(int create) {
       del_panel(card->level_panel); card->level_panel = NULL;
       del_panel(card->link_panel); card->link_panel = NULL;
       del_panel(card->noise_panel); card->noise_panel = NULL;
+      del_panel(card->bitrate_panel); card->bitrate_panel = NULL;
       continue;
     }
     if (card->flags & SHOW_LINK)
@@ -408,42 +514,87 @@ reset_panel(int create) {
     if (card->flags & SHOW_NOISE)
       create_panel(&(card->noise_panel),&(card->noise_text),NOISE_MAX,create);
     else { del_panel(card->noise_panel); card->noise_panel = NULL; }
+
+    if (card->flags & SHOW_BITRATE) {
+      if (get_bitrate(card,&mbitrate,&bitrate)) { 
+        create_panel(&(card->bitrate_panel),&(card->bitrate_text),
+                     mbitrate,create);
+      } else {
+        create_panel(&(card->bitrate_panel),&(card->bitrate_text),
+                     BITRATE_MAX,create);
+      }
+    }
+    else { del_panel(card->bitrate_panel); card->bitrate_panel = NULL; }
   }
 }
 
 static void
-update_panel(Panel *panel, Decal *decal
-               , char *header , char *ifname,float amount) {
-  Krell *k;
+update_panel(GkrellmPanel *panel, GkrellmDecal *decal,
+             char *ifname, char *amount_text, float amount) {
+  GkrellmKrell *k;
   char text[50];
 
   if (panel == NULL) return;
 
-  snprintf(text,sizeof(text),"%s: %.0f %s", ifname, amount, header);
+  if(ifname != NULL)
+    snprintf(text, sizeof(text), "%s: %s", ifname, amount_text);
+  else
+    strncpy(text, amount_text, sizeof(text));
   k = KRELL(panel);
   k->previous = 0;
-  gkrellm_update_krell(panel,k,amount);
+  gkrellm_update_krell(panel,k,abs(amount));
   gkrellm_draw_decal_text(panel,decal,text,amount);
-  gkrellm_draw_layers(panel);
+  gkrellm_draw_panel_layers(panel);
 }
 
 static void
-update_plugin (void) {
+update_normal_panel(GkrellmPanel *panel, GkrellmDecal *decal,
+                    char *header, char *ifname, float amount) {
+  char amount_text[50];
+  snprintf(amount_text, sizeof(amount_text), "%.0f %s", amount, header);
+  update_panel(panel, decal, ifname, amount_text, amount);
+}
+
+static void
+update_bitrate_panel(GkrellmPanel *panel, GkrellmDecal *decal,
+                     char *ifname, int maxbitrate, int bitrate) {
+  char amount_text[50];
+  float amount;
+
+  if(bitrate > 1e9)
+    snprintf(amount_text, sizeof(amount_text), "%.0f Gb/s", (bitrate / 1e9));
+  else if(bitrate > 1e6)
+    snprintf(amount_text, sizeof(amount_text), "%.0f Mb/s", (bitrate / 1e6));
+  else
+    snprintf(amount_text, sizeof(amount_text), "%.0f Kb/s", (bitrate / 1e3));
+  amount = (bitrate / (float)maxbitrate);
+  update_panel(panel, decal, ifname, amount_text, amount);
+}
+
+static void
+update_plugin(void) {
   wcard_t *card;
-  float link = 0, level = 0, noise =0;
+  float link = 0, level = 0, noise = 0;
+  int maxbitrate = 0, bitrate = 0;
+  char *ifname;
 
   if (GK.second_tick) {
     for (card = cards;  card != NULL; card = card->next) {
       if (!(card->flags & ACTIVE) || !(card->flags & SHOW)) continue;
+      ifname = ((card->flags & HIDE_NAME)?NULL:card->ifname);
       /* get_quality_info: get info in a system dependant way */  
-      if (!get_link_quality(card,&link,&level,&noise))  return;
-
-      update_panel(card->level_panel,card->level_text
-                     ,"Level",card->ifname,level);
-      update_panel(card->link_panel,card->link_text
-                     ,"Link",card->ifname,link);
-      update_panel(card->noise_panel,card->noise_text
-                     ,"Noise",card->ifname,noise);
+      if (get_link_quality(card,&link,&level,&noise)) {
+        update_normal_panel(card->level_panel,card->level_text
+                     ,"Level",ifname,level);
+        update_normal_panel(card->link_panel,card->link_text
+                     ,"Link",ifname,link);
+        update_normal_panel(card->noise_panel,card->noise_text
+                     ,"Noise",ifname,noise);
+      }
+      if (get_bitrate(card,&maxbitrate,&bitrate)) {
+        update_bitrate_panel(card->bitrate_panel,card->bitrate_text,
+                           ifname,maxbitrate,bitrate);
+      }
     }
   } 
   if (GK.two_second_tick) {
@@ -481,7 +632,7 @@ load_plugin_config(gchar *arg) {
   }
 }
 
-/* the config panel stuff and callbacs */
+/* the config panel stuff and callbacks */
 static void
 button_toggled(GtkWidget *widget, wcard_t *card, int value) {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
@@ -495,6 +646,11 @@ cb_show_button_toggled(GtkWidget *widget, wcard_t *card) {
   button_toggled(widget,card,SHOW);
 }
 
+static void
+cb_name_button_toggled(GtkWidget *widget, wcard_t *card) {
+  button_toggled(widget,card,HIDE_NAME);
+}
+
 
 static void
 cb_link_button_toggled(GtkWidget *widget, wcard_t *card) {
@@ -512,16 +668,22 @@ cb_noise_button_toggled(GtkWidget *widget, wcard_t *card) {
   button_toggled(widget,card,SHOW_NOISE);
 }
 
+static void
+cb_bitrate_button_toggled(GtkWidget *widget, wcard_t *card) {
+  button_toggled(widget,card,SHOW_BITRATE);
+}
+
 static void
 create_toggle_button(char *name,int active, wcard_t *card,
-                     GtkWidget *box,GtkSignalFunc func) {
+                     GtkWidget *box, GtkSignalFunc func) {
 
   GtkWidget *button;
 
   button = gtk_check_button_new_with_label(name);
   gtk_box_pack_start(GTK_BOX(box),button,FALSE,TRUE,3);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),active);
-  gtk_signal_connect(GTK_OBJECT(button),"toggled",func,card);
+  g_signal_connect(G_OBJECT(button),"toggled",
+      G_CALLBACK(func),card);
 }
 
 static void
@@ -539,18 +701,24 @@ create_device_tab(GtkWidget *notebook,wcard_t *card) {
   vbox = gtk_vbox_new(FALSE,0);
   gtk_container_add(GTK_CONTAINER(frame),vbox);
 
-  create_toggle_button("Show this Interface" ,card->flags & SHOW, card, 
-                         vbox,cb_show_button_toggled);
+  create_toggle_button("Show this interface" ,card->flags & SHOW, card, 
+                         vbox, (GtkSignalFunc) cb_show_button_toggled);
+  create_toggle_button("Hide interface name" ,card->flags & HIDE_NAME, card,
+                         vbox, (GtkSignalFunc) cb_name_button_toggled);
   
   separator = gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(vbox),separator,FALSE,TRUE,3);
 
   create_toggle_button("Show link quality" ,card->flags & SHOW_LINK, card, 
-                         vbox,cb_link_button_toggled);
+                         vbox, (GtkSignalFunc) cb_link_button_toggled);
   create_toggle_button("Show signal level" ,card->flags & SHOW_LEVEL, card, 
-                         vbox,cb_level_button_toggled);
+                         vbox, (GtkSignalFunc) cb_level_button_toggled);
   create_toggle_button("Show noise level" ,card->flags & SHOW_NOISE, card, 
-                         vbox,cb_noise_button_toggled);
+                         vbox, (GtkSignalFunc) cb_noise_button_toggled);
+#ifdef __linux__
+  create_toggle_button("Show bit rate" ,card->flags & SHOW_BITRATE, card, 
+                         vbox, (GtkSignalFunc) cb_bitrate_button_toggled);
+#endif /* __linux__ */
 }
 
 static void
@@ -567,28 +735,33 @@ static void
 create_help_text(GtkWidget *text) {
   gchar *info_text[] = 
   {
-    "<b>This plugin allows you to monitor the quality of a wireless lan card \n",
-    "<b>\nConfiguration:\n",
-    "Every detected wireless interface will have ", 
+    "<b>This plugin allows you to monitor the quality of a wireless lan card\n\n",
+    "<b>Configuration:\n",
+    "Every detected wireless interface will have ",
     "one config tab with the following options:\n",
-    "<b>\tShow this interface: ",
-    "\n\tShow information about the interface\n",
+    "<b>\tShow this interface:\n",
+    "\tShow information about the interface\n",
+    "<b>\tHide interface name:\n",
+    "\tHide interface name (e.g., eth1) from information\n",
     "<b>\tShow link quality:\n",
     "\tShow the link quality of this interface\n",
     "<b>\tShow signal level:\n",
     "\tShow the signal level of this interface\n",
     "<b>\tShow noise level:\n", 
-    "\tShow the noise level of this interface\n"
+    "\tShow the noise level of this interface\n",
+    "<b>\tShow bit rate:\n", 
+    "\tShow the bit rate of this interface\n"
   };
-  gkrellm_add_info_text(text,info_text,sizeof(info_text)/sizeof(gchar *));
+  gkrellm_gtk_text_view_append_strings(text,info_text,
+      sizeof(info_text)/sizeof(gchar *));
 }
 
 static void
 create_info_tab(GtkWidget *notebook) {
   GtkWidget *frame;
-  GtkWidget *label;
   GtkWidget *scrolled;
   GtkWidget *text;
+  GtkWidget *page;
 
   frame = gtk_frame_new(NULL);
   gtk_container_border_width(GTK_CONTAINER(frame),3);
@@ -597,13 +770,11 @@ create_info_tab(GtkWidget *notebook) {
         GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
   
   gtk_container_add(GTK_CONTAINER(frame),scrolled);
-  label = gtk_label_new("Info");
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),frame,label);
 
-  text = gtk_text_new(NULL,NULL);
-  gtk_text_set_editable(GTK_TEXT(text),FALSE);
+  page = gkrellm_gtk_notebook_page(notebook,"Info");
+  text = gkrellm_gtk_scrolled_text_view(page,NULL,
+                                   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   create_help_text(text);
-  gtk_container_add(GTK_CONTAINER(scrolled),text);
 }
 
 static void
@@ -651,7 +822,7 @@ apply_plugin_config(void) {
 }
 /* end of config panel stuff */
 
-static Monitor  wireless_meter  = {
+static GkrellmMonitor  wireless_meter  = {
   "Wireless",      /* Name, for config tab.    */
   0,          /* Id,  0 if a plugin       */
   create_plugin,    /* The create function      */
@@ -681,8 +852,9 @@ static Monitor  wireless_meter  = {
   /* All GKrellM plugins must have one global routine named init_plugin()
   |  which returns a pointer to a filled in monitor structure.
   */
-Monitor *
-init_plugin(void) {
+GkrellmMonitor *
+gkrellm_init_plugin(void) {
   panel_style_id = gkrellm_add_meter_style(&wireless_meter,"wireless");
-  return &wireless_meter;
+  monitor = &wireless_meter;
+  return monitor;
 }
