@@ -18,10 +18,17 @@
 |  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <gkrellm/gkrellm.h>
+#include <gkrellm2/gkrellm.h>
 #ifdef __FreeBSD__
+  #include <sys/types.h>
+  #include <sys/socket.h>
+  #include <sys/sockio.h>
   #include <net/if.h>
-  #include <machine/if_wavelan_ieee.h>
+  #if __FreeBSD_version >= 470000
+    #include <dev/wi/if_wavelan_ieee.h>
+  #else
+    #include <machine/if_wavelan_ieee.h>
+  #endif
   #define ANCACHE
   #include <dev/an/if_aironet_ieee.h>
 #endif
@@ -35,23 +42,33 @@
   #include <dev/pcmcia/if_wi_ieee.h>
 #endif
 
+#ifdef __linux__
+  #include <netinet/in.h>
+  #include <linux/netdevice.h>
+  #include <linux/wireless.h>
+  #include <sys/ioctl.h>
+#endif
+
 #define PLUGIN_CONFIG_KEYWORD "gkrellmwireless"
 
-#define WIRELESS_MAJOR_VERSION 0
-#define WIRELESS_MINOR_VERSION 2
+#define WIRELESS_MAJOR_VERSION 2
+#define WIRELESS_MINOR_VERSION 0
 #define WIRELESS_EXTRA_VERSION ".2"
 
 /* flags for all the differenct stuff */
-#define ACTIVE     (1 << 0) /* this interface is alive now */ 
-#define SHOW       (1 << 1) /* show or don't show this interface */
-#define SHOW_LINK  (1 << 2) /* show link quality of this interface */
-#define SHOW_LEVEL (1 << 3) /* show level of this interface */ 
-#define SHOW_NOISE (1 << 4) /* show noise on this interface */
+#define ACTIVE       (1 << 0) /* this interface is alive now */ 
+#define SHOW         (1 << 1) /* show or don't show this interface */
+#define SHOW_LINK    (1 << 2) /* show link quality of this interface */
+#define SHOW_LEVEL   (1 << 3) /* show level of this interface */ 
+#define SHOW_NOISE   (1 << 4) /* show noise on this interface */
+#define SHOW_BITRATE (1 << 5) /* show bit rate on this interface */
+#define HIDE_NAME    (1 << 6) /* show name of this interface */
 
 /* some maximums */
-#define LINKQ_MAX 100
-#define LEVEL_MAX 255
-#define NOISE_MAX 255
+#define LINKQ_MAX 255
+#define LEVEL_MAX 256
+#define NOISE_MAX 256
+#define BITRATE_MAX 1
 
 typedef struct __wcard_t wcard_t;
 
@@ -61,11 +78,13 @@ struct __wcard_t {
   gint  flags; /* configuration flags */
   gint cflags;
 
-  Panel *level_panel;
-  Panel *link_panel;
-  Panel *noise_panel;
+  GkrellmPanel *level_panel;
+  GkrellmPanel *link_panel;
+  GkrellmPanel *noise_panel;
+  GkrellmPanel *bitrate_panel;
 
-  Decal *level_text;
-  Decal *link_text;
-  Decal *noise_text;
+  GkrellmDecal *level_text;
+  GkrellmDecal *link_text;
+  GkrellmDecal *noise_text;
+  GkrellmDecal *bitrate_text;
 };
